package com.sleepycat.je.tree;

import com.sleepycat.je.dbi.DatabaseImpl;

public privileged aspect DINBINBudget {

	after(DIN din):
		execution(DIN.new(DatabaseImpl, byte[], int, byte[], ChildReference, int)) 
		&& this(din) 
	{
		din.initMemorySize(); // MB// init after adding Dup Count LN. */
	}

	before(DIN din, ChildReference dupCountLNRef):
		execution(void DIN.setDupCountLN(ChildReference)) && args(dupCountLNRef) && this(din) {

		din.updateMemorySize(din.dupCountLNRef, dupCountLNRef);
	}

	void around(DIN din):
		(execution(void DIN.updateDupCountLN(Node)) ||
		execution(void DIN.updateDupCountLNRefAndNullTarget(long))) && 
		this(din){

		long oldSize = din.getEntryInMemorySize(din.dupCountLNRef.getKey(),
				din.dupCountLNRef.getTarget());
		proceed(din);
		long newSize = din.getEntryInMemorySize(din.dupCountLNRef.getKey(),
				din.dupCountLNRef.getTarget());
		din.updateMemorySize(oldSize, newSize);// MB

	}
	
	before(int index, BIN bin):
		execution(void BIN.setKnownDeleted(int)) && args(index) && this(bin) {
		
		bin.updateMemorySize(bin.getTarget(index), null);//MB
	}
	
	after(BIN bin) returning(long removed):
		execution(long BIN.evictLNs()) && this(bin) {

		bin.updateMemorySize(removed, 0);//MB
	}
	
	after(BIN bin)returning(long removed):
		call(long BIN.evictInternal(..)) && withincode(* BIN.evictLN(..)) && this(bin) {
		
		bin.updateMemorySize(removed, 0);//MB
	}
	
	Node around(IN in, ChildReference cr):
		execution(Node ChildReference.fetchTarget(DatabaseImpl, IN)) &&
		args(DatabaseImpl,in) && this(cr) && within(ChildReference) 
	{
		
		boolean wasNull = cr.target!=null;
		
		Node result= proceed(in,cr);
		
		if (wasNull){
			if (in != null) {
				in.updateMemorySize(null, cr.target);//MB
			}
		}
		
		return result;
	}
}
