package com.sleepycat.je.tree;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.DbConfigManager;
import com.sleepycat.je.dbi.MemoryBudget;

public privileged aspect WeaveTreeBudget {
	/* Called once at environment startup by MemoryBudget. */
	public static long DIN.computeOverhead(DbConfigManager configManager)
			throws DatabaseException {

		/*
		 * Overhead consists of all the fields in this class plus the entry
		 * arrays in the IN class.
		 */
		return MemoryBudget.DIN_FIXED_OVERHEAD
				+ IN.computeArraysOverhead(configManager);
	}

	long DIN.getMemoryOverhead(MemoryBudget mb) {
		return mb.getDINOverhead();
	}
	
	/* Called once at environment startup by MemoryBudget */
	public static long BIN.computeOverhead(DbConfigManager configManager)
			throws DatabaseException {//MB

		/*
		 * Overhead consists of all the fields in this class plus the entry
		 * arrays in the IN class.
		 */
		return MemoryBudget.BIN_FIXED_OVERHEAD
				+ IN.computeArraysOverhead(configManager);
	}

	long BIN.getMemoryOverhead(MemoryBudget mb) {//MB
		return mb.getBINOverhead();
	}
	
	/**
	 * Count up the memory usage attributable to this node alone.
	 */
	long DBIN.computeMemorySize() {//MB
		long size = super.computeMemorySize();
		/*
		 * XXX Need to update size when changing the dupKey. if (dupKey != null &&
		 * dupKey.getKey() != null) { size +=
		 * MemoryBudget.byteArraySize(dupKey.getKey().length); }
		 */
		return size;
	}

	/* Called once at environment startup by MemoryBudget. */
	public static long DBIN.computeOverhead(DbConfigManager configManager)
			throws DatabaseException {//MB

		/*
		 * Overhead consists of all the fields in this class plus the entry
		 * arrays in the IN class.
		 */
		return MemoryBudget.DBIN_FIXED_OVERHEAD
				+ IN.computeArraysOverhead(configManager);
	}

	long DBIN.getMemoryOverhead(MemoryBudget mb) {//MB
		return mb.getDBINOverhead();
	}
	
	/**
	 * Count up the memory usage attributable to this node alone. LNs children
	 * are counted by their BIN/DIN parents, but INs are not counted by their
	 * parents because they are resident on the IN list.
	 */
	long DIN.computeMemorySize() {//MB
		long size = super.computeMemorySize();
		if (dupCountLNRef != null) {
			size += getEntryInMemorySize(dupCountLNRef.getKey(), dupCountLNRef
					.getTarget());
		}
		/*
		 * XXX Need to update size when changing the dupKey. if (dupKey != null &&
		 * dupKey.getKey() != null) { size +=
		 * MemoryBudget.byteArraySize(dupKey.getKey().length); }
		 */
		return size;
	}
	
	/**
	 * Compute the approximate size of this node in memory for evictor
	 * invocation purposes.
	 */
	public long DupCountLN.getMemorySizeIncludedByParent() {
		return MemoryBudget.DUPCOUNTLN_OVERHEAD;
	}//MB
}
