package com.sleepycat.je.txn;

import java.util.Collections;
import java.util.Set;

import com.sleepycat.je.dbi.MemoryBudget;

public privileged aspect LockBudget {
	private static final int REMOVE_LOCKINFO_OVERHEAD = 0 - MemoryBudget.LOCKINFO_OVERHEAD;// MB

	private MemoryBudget Lock.memoryBudget = null;

	void Lock.setMemoryBudget(MemoryBudget memoryBudget) {
		this.memoryBudget = memoryBudget;
	}

	after(int lockTableIndex, Lock lock):
		(
				execution(void Lock.addWaiterToEndOfList(LockInfo, int)) || 
				execution(void Lock.addWaiterToHeadOfList(LockInfo, int)) ||
				execution(void Lock.addOwner(LockInfo, int))
				
		) && args(LockInfo,lockTableIndex) && this(lock){
		lock.memoryBudget.updateLockMemoryUsage(MemoryBudget.LOCKINFO_OVERHEAD,
				lockTableIndex);// MB

	}

	// made homogenic myself by chaning return values
	after(int lockTableIndex, Lock lock)returning (boolean removed):
		(
				execution(boolean Lock.flushWaiter(Locker, int)) ||
				execution(boolean Lock.flushOwner(LockInfo, int))
		) && args(*,lockTableIndex) && this(lock){

		if (removed) {
			lock.memoryBudget.updateLockMemoryUsage(REMOVE_LOCKINFO_OVERHEAD,
					lockTableIndex);// MB
		}
	}

	// made homogenic myself by chaning return values
	after(int lockTableIndex, Lock lock)returning (LockInfo flushedInfo):
		execution(LockInfo Lock.flushOwner(Locker,int)) &&
		args(Locker,lockTableIndex) && this(lock){

		if (flushedInfo != null) {
			lock.memoryBudget.updateLockMemoryUsage(REMOVE_LOCKINFO_OVERHEAD,
					lockTableIndex);// MB
		}
	}

	after(int lockTableIndex, Lock lock)returning (Set result):
		execution(Set Lock.release(Locker,int)) &&
		args(Locker,lockTableIndex) && this(lock){

		if (result != Collections.EMPTY_SET) {
			lock.memoryBudget.updateLockMemoryUsage(result.size()
					* REMOVE_LOCKINFO_OVERHEAD, lockTableIndex);// MB
		}
	}

	before(int numRemovedLockInfos, int lockTableIndex, Lock lock):
		execution(void Lock.hook_removedLockInfos(int,int)) 
		&& args(numRemovedLockInfos,lockTableIndex) && this(lock) 
	{

		lock.memoryBudget.updateLockMemoryUsage(
				0 - (numRemovedLockInfos * MemoryBudget.LOCKINFO_OVERHEAD),
				lockTableIndex);// MB
	}

	after(LockManager lockManager) returning (Lock newLock):
		call(Lock.new(Long)) && this(lockManager){

		newLock.setMemoryBudget(lockManager.memoryBudget);
	}

	declare error:call(Lock.new(..)) && !within(LockManager):"cannot instanciate Lock class outside LockManager (needs setMemoryManager() call)";
}
