package com.sleepycat.je.txn;

import java.util.Map;

import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.dbi.MemoryBudget;
import com.sleepycat.je.txn.LockManager.LockTable;

public privileged aspect LockManagerBudget {
	/*
	 * The total memory cost for a lock is the Lock object, plus its entry and
	 * key in the lock hash table.
	 * 
	 * The addition and removal of Lock objects, and the corresponding cost of
	 * their hashmap entry and key are tracked through the LockManager.
	 */
	static final long TOTAL_LOCK_OVERHEAD = MemoryBudget.LOCK_OVERHEAD// MB
			+ MemoryBudget.HASHMAP_ENTRY_OVERHEAD + MemoryBudget.LONG_OVERHEAD;

	private static final long REMOVE_TOTAL_LOCK_OVERHEAD = 0 - TOTAL_LOCK_OVERHEAD;

	private MemoryBudget LockManager.memoryBudget;// MB
	
	after(LockManager lm):set(EnvironmentImpl LockManager.envImpl) && target(lm){
		lm.memoryBudget = lm.envImpl.getMemoryBudget();// MB
	}
	
	after(LockManager lm, LockTable lockTable):
		call(Object Map.put(Object, Object)) && target(lockTable) && this(lm) {
		
		lm.memoryBudget.updateLockMemoryUsage(TOTAL_LOCK_OVERHEAD,
				lockTable.lockTableIndex);// MB
	}
	after(LockManager lm, LockTable lockTable):
		call(Object Map.remove(Object)) && target(lockTable) && this(lm) {
		
		lm.memoryBudget.updateLockMemoryUsage(REMOVE_TOTAL_LOCK_OVERHEAD,
				lockTable.lockTableIndex);// MB
	}

}
